@@ -343,6 +343,8 @@ def get_update_project_args(args):
         input_params["downloadRestricted"] = True if args.download_restricted == 'true' else False
     if args.containsPHI is not None:
         input_params["containsPHI"] = True if args.containsPHI == 'true' else False
+    if args.database_ui_view_only is not None:
+        input_params["databaseUIViewOnly"] = True if args.database_ui_view_only == 'true' else False
     if args.bill_to is not None:
         input_params["billTo"] = args.bill_to
     if args.allowed_executables is not None:
